/*  Copyright (C) 2002-2013 Zoltán Nagy
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef __HEP_MATRIX_H__
#define __HEP_MATRIX_H__ 1

#include <vector>


namespace duct {
  
  template<typename _Tp, class _Alloc = std::allocator<_Tp>>
  class matrix : public std::vector<_Tp, _Alloc>
  {
    typedef std::vector<_Tp, _Alloc> _Base;
    
  public: 
    typedef typename _Base::size_type size_type;
    typedef typename _Base::value_type value_type;
    typedef typename _Base::iterator iterator;
    typedef typename _Base::pointer pointer;
    typedef typename _Base::reference reference;
    typedef typename _Base::reverse_iterator reverse_iterator;
    typedef typename _Base::const_iterator const_iterator;
    typedef typename _Base::const_reference const_reference;
    typedef typename _Base::const_pointer const_pointer;
    typedef typename _Base::const_reverse_iterator const_reverse_iterator;
    typedef typename _Base::allocator_type allocator_type;
    
    //----- default constructor (creates empty matrix) -----
    explicit matrix(const _Tp& c = _Tp(), const _Alloc& a = _Alloc()) : _Base(0,c), _M_nrow(0), _M_ncol(0) {}
    
    //----- creating an n x m matrix 
    matrix(size_type n, size_type m, const _Tp& c = _Tp(), const _Alloc& a = _Alloc()) 
    : _Base(n*m,c), _M_nrow(n), _M_ncol(m) {}
    
    //----- destructor -----
    ~matrix() = default;
    
    //----- copy operations -----
    matrix(const matrix&) = default;
    matrix(matrix&&) = default;
    
    //----- move operations -----
    matrix& operator=(const matrix&) = default;
    matrix& operator=(matrix&&) = default;
    
    //----- resizing the matrix -----
    void resize(size_type n, size_type m, const _Tp& val = _Tp()) {
      _M_nrow = n; _M_ncol = m;
      _Base::resize(n*m, val);
    }
    
    /*  Element access: the return of these functions points to
     *  the first element of the kth row. To get an element of the 
     *  matrix, one should do just m[r][c].
     */
    pointer operator[](size_type k) { return _Base::data()+_M_ncol*k;}
    const_pointer operator[](size_type k) const { return _Base::data()+_M_ncol*k;}
    
    /* The size of the matrix */
    size_type nrow() const { return _M_nrow;}
    size_type ncol() const { return _M_ncol;}
    
  private:
    //----- data members -----
    size_type _M_nrow, _M_ncol;
  };

  
}



#endif
