/*  Copyright (C) 2013 Zoltán Nagy
 *  Created by Zoltán Nagy on 3/13/13.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef __mc_splitting_functions_h__
#define __mc_splitting_functions_h__ 1

//   std includes

//   project headers



namespace duct {

  
  /*  We use a separate namespace for the splitting function.
   *  Of course it is not necessary but I like to hide things
   *  what a simple user doesn't want to know...
   */
  namespace __mc_splitting_functions
  {
    /* This is the splitting function in terms of the splitting variables
     * for the f-> f + g processes. This is very genereral and it requires
     * some adjustment when we build the shower engines.
     */
    struct emitting_gluon_fin
    {
      //----- The function and its integral -----
      void caching(int, double, double, double);
      void caching(int, int, double, double, double, double, double, double);
      double function_phi(double) const;
      double primitive_phi(double) const;
      
      //----- function after the phi integral ------
      double operator()(int sl, char spectator, double rl, double ymax, double xi, double mu, double y, double u)
      {
        this -> caching(sl, spectator, rl, ymax, xi, mu, y, u);
        return this -> primitive_phi(1.0);
      }

      double operator()(int sl, double ymax, double y, double u)
      {
        this -> caching(sl, ymax, y, u);
        return this -> primitive_phi(1.0);
      }
      
    private:
      //----- parameters -----
      unsigned int isSpec;
      double A, B, C, D, S, I1, I2, R1, R2, Hcoll, F;
    };
    
    
    /*  This is the splitting function of the f -> f + g process,
     *  when the spectator is a massive incoming parton (like top)
     *  in the decay shower. This is simple and we have a function only.
     */
    double emitting_gluon_dcy(int, double, double, double, double);
    
    
    /*  This is the splitting function for g -> f + fbar process.
     *  This is very genereral and it requires some adjustment
     *  when we build the shower engines.
     */
    double gluon_splitting_fin(double, double, double, double);
    
    
    /*  This class represents the f -> f+g splitting in the initial state. 
     *  Like in the final state the azimuthal dependence is not that simple
     *  thus it is simpler to have a class representation of this function.
     *  This is very general and requires some adjustment when we build the 
     *  shower engine.
     */
    struct emitting_gluon_ini {
      //----- The function and its integral -----
      void caching(int, int, double, double, double, double, double, double);
      double function_phi(double) const;
      double primitive_phi(double) const;
      
      //----- function after the phi integral ------
      double operator()(int sl, int spec, double ma, double mb, double xi, double xT, double y, double z) {
        this -> caching(sl, spec, ma, mb, xi, xT, y, z);
        return this -> primitive_phi(1.0);
      }
      
    private:
      //----- cached variables -----
      bool isSpec;
      double w1, w2, w3, R1, R2, R3, R4, R5, R6;
      double Ap, Am, Bp, Bm, Cp, Cm, rAp, rAm, rCp, rCm, Hcoll;
    };
    

    /*  This is the splitting function of the q -> g + q in the initial state.
     *  It is just a simple function. 
     */
    double splitting_gluon_quark_ini(double, double, double, double);

  
    /*  This is the splitting function of the g -> q + qbar in the initial state.
     *  It is just a simple function.
     */
    double splitting_quark_antiquark_ini(double, double, double, double);
    
  
  }   //  namespace __mc_splitting_functions
}   //  namespace nlo

#endif








