/*  Copyright (C) 2014 Zoltán Nagy
 *  Created by Zoltán Nagy on 1/5/14.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef __nlo_cxx_support_h__
#define __nlo_cxx_support_h__ 1

/*   std includes   */
#include <string>
#include <iostream>
#include <mutex>


/*   project headers   */


namespace duct {
  
  /*    binary write of a string  */
  inline std::ostream& write_bin(std::ostream& os, const std::string& ltx)
  {
    unsigned s = ltx.size();
    os.write(reinterpret_cast<const char *>(&s), sizeof(unsigned));
    os.write(ltx.data(), s*sizeof(char));
    return os;
  }
  
  /*  binary read of a string   */
  inline std::istream& read_bin(std::istream& is, std::string& ltx)
  {
    unsigned s;
    is.read(reinterpret_cast<char *>(&s), sizeof(unsigned));
    ltx.resize(s);
    for(unsigned k = 0; k < s; ++k)
      ltx[k] = is.get();
  
    return is;
  }

  /*   write string as text  */
  inline std::ostream& write_txt(std::ostream& os, const std::string& ltx) {
    os<<ltx.size()<<" "<<ltx;
    return os;
  }
  
  /*   read string back as text  */
  inline std::istream& read_txt(std::istream& is, std::string& ltx)
  {
    unsigned s;
    is>>s; is.get();
    
    ltx.resize(s);
    for(unsigned k = 0; k < s;++k)
      ltx[k] = is.get();
    
    return is;
  }

  
  /*   locks for std::cout, std::cerr, std::cin   */
  extern std::mutex mtx_cout, mtx_cerr, mtx_cin;
}

#endif
