/*  Copyright (C) 2010-2013 Zoltán Nagy
 *  Created by Zoltan Nagy on 10/5/10.
 *  
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>. 
 */
#ifndef __NLO_MAPPING_H__
#define __NLO_MAPPING_H__ 1


//   standard c++ includes

//   nlojet++ includes
#include "bits/nlo-event.h"


namespace duct {
  
    
  template<class _Tag>
  struct splitting_variables;
  
  
  /*  This class represent a set of variables those we need to
   *  generate standard shower splittings in the final state. 
   */
  template<>
  struct splitting_variables<lo_shower> 
  {
    //----- label of the emitter and the possible spectator ----
    int emitter, spectator;
    
    //----- flavor splitting variable -----
    struct zeta_flavor {
      int key1, key2;
    } zeta;
    
    //----- parametrization of the new momenta
    double y, z, phi;
    
    //----- paramaters of the color operator -----
    int sigma, sigmaP;
  };
  
  
  /*  This function does the mapping in the color, flavor and momentum space. 
   *  The function changes the the first argument.
   */
  template<class _TagE, class _TagS>
  void mapping(event<_TagE>&, const event<_TagE>&, const splitting_variables<_TagS>&);
  
  
  /*  Inverse mapping. This is needed for NLO calculations.
   *  This functions returns the event with one less parton 
   *  and splitting variables.
   */
  
    
}
#endif

