/*  Copyright (C) 2013 Zoltán Nagy
 *  Created by Zoltán Nagy on 6/25/13.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef __nlo_tags_h__
#define __nlo_tags_h__ 1

//   std includes

//   project headers


namespace duct {
  
  /*  Empty classes to tag ranges in the class statistical_basis. 
   *  With these we can get some information in a more elegant way.
   */
  struct qcd_out {};
  struct non_qcd {};
  struct incomings {};
  struct outgoings {};

  /*   Tagging the splitting variarbles   */
  struct lo_shower {};
  struct mpi {};

  
  /* Tagging the classes those are related to e+e- anniilation. */
  struct epa {
    static constexpr const char *name = "epa";
  };
  
  /* Tagging the classes those are related to DIS. */
  struct dis {
    static constexpr const char *name = "dis";
  };
  
  /* Tagging the classes those are related to hadron-hadron collision. */
  struct hhc {
    static constexpr const char *name = "hhc";
  };
  
  /* Tagging the classes those are related to heavy particle decay. */
  struct decay {
    static constexpr const char *name = "decay";
  };

  /*  Tagging classes those are related to the initial or final state. 
   *  For example: splitting_kernel<ini>
   */
  struct ini {};
  struct fin {};
  
  /*   simple class to tag the incoming hadrons of the hard processes  */
  template<class _Tag>
  struct incoming_hadrons {};
  
  template<>
  struct incoming_hadrons<dis> {
    explicit incoming_hadrons(int h = 2212) : hadron(h) {}
    int hadron;
  };
  
  template<>
  struct incoming_hadrons<hhc> {
    explicit incoming_hadrons(int h = 2212) : hadronA(h), hadronB(h) {}
    incoming_hadrons(int h1, int h2) : hadronA(h1), hadronB(h2) {}
    int hadronA, hadronB;
  };

  
  /*  Tagging classes those have dependence on the type of the ordering   */
  struct ordering {
    struct lambda {
      static constexpr const char *name = "lambda";
    };
    
    struct lambdaT {
      static constexpr const char *name = "lambdaT";
    };
    
    struct kT {
      static constexpr const char *name = "kT";
    };
    
    struct angular {
      static constexpr const char *name = "angular";
    };
  };
  
  /*   MSbar tag   */
  struct msbar {};
  
  /* Index class to implement such an operator like
   *  event<hhc> p;
   *  lorentzvector<double> pA;
   *
   *  p[hadron(-1)].key = key;
   *  p[hadron(-1)].momentum = pA;
   */
  struct hadron {
    explicit hadron(int k = 0) : idx(k) {}
    int idx;
  };
  
  
  /*   In this code everything is expressed in GeV,
   *   as inpunt we might want to give some quantities in TeV, MeV.
   */
  inline constexpr double operator"" _MeV(long double s) { return s/1000.0;}
  inline constexpr double operator"" _GeV(long double s) { return s;}
  inline constexpr double operator"" _TeV(long double s) { return 1000.0*s;}
  
  inline constexpr double operator"" _MeV(unsigned long long s) { return s/1000.0;}
  inline constexpr double operator"" _GeV(unsigned long long s) { return s;}
  inline constexpr double operator"" _TeV(unsigned long long s) { return 1000.0*s;}
  
}

#endif
