/*  Copyright (C) 2013 Zoltán Nagy
 *  Created by Zoltán Nagy on 7/12/13.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

/*   std includes    */
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <errno.h>


/*   project headers   */
#include <bits/nlo-c-support.h>




int __nlo_helper_mkdir(const char *path, int mode)
{
  struct stat sb;
  
  if(!stat(path, &sb)) {
    if(S_ISDIR (sb.st_mode) == 0) {
      fprintf(stderr, "`%s': file exists but is not a directory\n", path);
      return 1;
    }
    
    return 0;
  }
  
  char *npath = strdup(path);
  char *p = npath;
  
  /* Skip leading slashes. */
  while(*p == '/') p++;
  
  while((p = strchr(p, '/')))
  {
    *p = '\0';
    if(stat(npath, &sb)) {
      if(mkdir(npath, mode))
      {
        fprintf(stderr, "cannot create directory `%s': %s\n", npath, strerror(errno));
        free(npath);
        return 1;
      }
    } else if(!S_ISDIR(sb.st_mode)) {
      fprintf(stderr, "`%s': file exists but is not a directory\n", npath);
      free(npath);
      return 1;
    }
    
    *p++ = '/';   /* restore slash */
    while(*p == '/') p++;
  }
  
  /* Create the final directory component. */
  if(stat(npath, &sb) && mkdir(npath, mode))
  {
    fprintf(stderr, "cannot create directory `%s': %s\n", npath, strerror(errno));
    free(npath);
    return 1;
  }
  
  free(npath);
  return 0;
}


