/*  Copyright (C) 2012-2013 Zoltn Nagy
 *  Created by Zoltn Nagy on 14/2/12.
 *  
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program. If not, see <http://www.gnu.org/licenses/>. 
 */

//   standard c++ includes

//   project includes
#include "bits/mc-emission.h"
#include "bits/qcd-color.h"
#include "bits/nlo-mapping.h"


namespace duct {
  
 
  
  double selecting_color_operator(random_engine_type& rne, splitting_variables<lo_shower> *vars, const statistical_basis& p)
  {
    const model *mdl = p.get_model();
    int fl = vars->zeta.key1, fm = vars->zeta.key2;
    int cl = mdl->color(fl), cm = mdl->color(fm);
    
    //----- default weight -----
    double weight = 1.0;
    
    //  spliting e.g.: g -> g + g
    if(cl == 8 && cm == 8)
    {
      /* First we have to calculate the probabilities of the operators.   */
      unsigned int chi_p = 0U, chi_m = 0U, chiP_p = 0U, chiP_m = 0U;
      
      if(p[vars->emitter].color.next == vars->spectator) ++chi_p;
      else if(p[vars->emitter].color.prev == vars->spectator) ++chi_m;
      
      if(p[vars->emitter].colorP.next == vars->spectator) ++chiP_p;
      else if(p[vars->emitter].colorP.prev == vars->spectator) ++chiP_m;
      
      unsigned int chi_pp = chi_p + chiP_p, chi_mm = chi_m + chiP_m;
      unsigned int chi_pm = chi_p + chiP_m, chi = chi_pp + chi_mm;
      
      //----- defining the probabilities ------
      std::uniform_real_distribution<double> rng(0.0,1.0);
      
      //----- frail gluon splits ---
      if(p[vars->emitter].health == 'f') {
        //----- commulative probabilities -----
        double cp[4] = {(0.5*chi_pp)/chi, 0.5, 0.5 + (0.5*chi_pm)/chi, 1.0};
          
        //----- selecting the operator -----
        double rn = rng(rne);
        int n = 0; while(cp[n] < rn) n++;
        
        switch(n) {
          case 0: vars->sigma =  1; vars->sigmaP =  1; break;
          case 1: vars->sigma = -1; vars->sigmaP = -1; break;
          case 2: vars->sigma =  1; vars->sigmaP = -1; break;
          case 3: vars->sigma = -1; vars->sigmaP =  1; break;
        }
        
        //----- return the weight -----
        weight = 2.0*Na/Nc2;
        if(vars->sigma != vars->sigmaP) weight *= -1;
      }
      
      //----- healthy gluon splits -----
      else {
        //----- commulative probabilities -----
        double pp = Na/Nc2;
        double cp[4] = {(pp*chi_pp)/chi, pp, pp + ((1.0-pp)*chi_pm)/chi, 1.0};
        
        //----- selecting the operator -----
        double rn = rng(rne);
        int n = 0; while(cp[n] < rn) n++;
        
        switch(n) {
          case 0: vars->sigma =  1; vars->sigmaP =  1; break;
          case 1: vars->sigma = -1; vars->sigmaP = -1; break;
          case 2: vars->sigma =  1; vars->sigmaP = -1; break;
          case 3: vars->sigma = -1; vars->sigmaP =  1; break;
        }
        
        //----- For crossed gluon configuration the weight is 1-Nc^2 -----
        if(vars->sigma != vars->sigmaP) weight = -Na;
      }
    }
    
    //   splitting e.g.: q -> q + g
    else if(cl == 3 && cm == 8) vars->sigma = vars->sigmaP = 1;
    
    //   splitting e.g.: q~ -> q~ + g
    else if(cl == -3 && cm == 8) vars->sigma = vars->sigmaP = -1;
    
    //  spliting e.g.: g -> q + q~
    else if(std::abs(cl) == 3 && cm == -cl)
    {
      int sigma = (cl == 3 ? 2 : 3);
      
      /*   If it is a frail gluon   */
      if(p[vars->emitter].health == 'f') {
        //----- commulative probabilities -----
        double cp[4] = {0.25, 0.5, 0.75, 1.0};
        
        //----- selecting the operator -----
        std::uniform_real_distribution<double> rng(0.0,1.0);
        double rn = rng(rne);
        int n = 0; while(cp[n] < rn) n++;
        
        //----- overall weight -----
        weight = 4.0/Na;
        
        switch(n) {
          case 0: vars->sigma =  sigma; vars->sigmaP =  sigma; weight *= Nc2; break;
          case 1: vars->sigma =  sigma; vars->sigmaP = -sigma; weight *= -Nc; break;
          case 2: vars->sigma = -sigma; vars->sigmaP =  sigma; weight *= -Nc; break;
          case 3: vars->sigma = -sigma; vars->sigmaP = -sigma; break;
        }
      }
      
      /*    Healthy gluon emitter   */
      else {
        //----- commulative probabilities -----
        constexpr double pp = 1.0/(Nc2+3.0);
        double cp[4] = {pp*Nc2, pp*(Nc2+1.0), pp*(Nc2+2.0), 1.0};
        
        //----- selecting the operator -----
        std::uniform_real_distribution<double> rng(0.0,1.0);
        double rn = rng(rne);
        int n = 0; while(cp[n] < rn) n++;
        
        //----- overall weight -----
        weight = (Nc2+3.0)/Na;
        
        switch(n) {
          case 0: vars->sigma =  sigma; vars->sigmaP =  sigma; break;
          case 1: vars->sigma =  sigma; vars->sigmaP = -sigma; weight *= -Nc; break;
          case 2: vars->sigma = -sigma; vars->sigmaP =  sigma; weight *= -Nc; break;
          case 3: vars->sigma = -sigma; vars->sigmaP = -sigma; break;
        }
      }
    }
    
    //  splitting e.g.: q -> g + q in the initial state
    else if(cl == 8 && cm == 3) vars->sigma = vars->sigmaP = 4;
    
    //  splitting e.g.: q~ -> g + q~ in the initial state
    else if(cl == 8 && cm == -3) vars->sigma = vars->sigmaP = -4;
    
    //   unkown splitting
    else throw "selecting_color_operator : unable to assign the new color state";
    
    return weight;
  }
  

  double selecting_color_operator_u3(random_engine_type& rne, splitting_variables<lo_shower> *vars, const statistical_basis& p)
  {
    const model *mdl = p.get_model();
    int fl = vars->zeta.key1, fm = vars->zeta.key2;
    int cl = mdl->color(fl), cm = mdl->color(fm);
    
    //----- default weight -----
    double weight = 1.0;
    
    //  spliting e.g.: g -> g + g
    if(cl == 8 && cm == 8)
    {
      /* First we have to calculate the probabilities of the operators.   */
      unsigned int chiP = 0U, chiM = 0U;
      
      if(p[vars->emitter].color.next == vars->spectator) ++chiP;
      else if(p[vars->emitter].color.prev == vars->spectator) ++chiM;
      
      if(p[vars->emitter].colorP.next == vars->spectator) ++chiP;
      else if(p[vars->emitter].colorP.prev == vars->spectator) ++chiM;
      
      /*   selecting the color operator    */ 
      std::uniform_real_distribution<double> rng(0.0,1.0);
      
      if((chiP+chiM)*rng(rne) < static_cast<double>(chiP)) vars->sigma = vars->sigmaP = 1;
      else vars->sigma = vars->sigmaP = -1;
    }
    
    //   splitting e.g.: q -> q + g
    else if(cl == 3 && cm == 8) vars->sigma = vars->sigmaP = 1;
    
    //   splitting e.g.: q~ -> q~ + g
    else if(cl == -3 && cm == 8) vars->sigma = vars->sigmaP = -1;
    
    //  spliting e.g.: g -> q + q~
    else if(cl ==  3 && cm == -cl) vars->sigma = vars->sigmaP = 2;
    else if(cl == -3 && cm == -cl) vars->sigma = vars->sigmaP = 3;
    
    //  splitting e.g.: q -> g + q in the initial state
    else if(cl == 8 && cm == 3) vars->sigma = vars->sigmaP = 4;
    
    //  splitting e.g.: q~ -> g + q~ in the initial state
    else if(cl == 8 && cm == -3) vars->sigma = vars->sigmaP = -4;
    
    //   unkown splitting
    else throw "selecting_color_operator_u3 : unable to assign the new color state";
    
    return weight;
  }
  
  

}    //   namespace nlo
