/*  Copyright (C) 2013 Zoltán Nagy
 *  Created by Zoltán Nagy on 4/24/13.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

//   std includes

//   project headers
#include "bits/qcd-color.h"
#include "bits/mc-engine.h"


namespace duct {
  namespace __mc_helper {
    
    unsigned int counting_color_connections(const statistical_basis& p, int emitter, std::pair<int, double> *spectator)
    {
      const model *mdl = p.get_model();
      unsigned int ns;
      double cfac = 0.5*Cf;
      
      //----- possible spectators ----
      switch(mdl->color(p[emitter].flavor)) {
        case 8:
          ns = 2;
          cfac = 0.25*Ca;
          spectator[0] = {p[emitter].color.prev, cfac};
          spectator[1] = {p[emitter].color.next, cfac};
          
          if(p[emitter].colorP.prev == spectator[0].first) spectator[0].second += cfac;
          else if(p[emitter].colorP.prev == spectator[1].first) spectator[1].second += cfac;
          else spectator[ns++] = {p[emitter].colorP.prev, cfac};
          
          if(p[emitter].colorP.next == spectator[0].first) spectator[0].second += cfac;
          else if(p[emitter].colorP.next == spectator[1].first) spectator[1].second += cfac;
          else spectator[ns++] = {p[emitter].colorP.next, cfac};
          break;
        case 3:
          ns = 1;
          spectator[0] = {p[emitter].color.next, cfac};
          
          if(p[emitter].colorP.next == spectator[0].first) spectator[0].second += cfac;
          else spectator[ns++] = {p[emitter].colorP.next, cfac};
          break;
        case -3:
          ns = 1;
          spectator[0] = {p[emitter].color.prev, cfac};
          
          if(p[emitter].colorP.prev == spectator[0].first) spectator[0].second += cfac;
          else spectator[ns++] = {p[emitter].colorP.prev, cfac};
          break;
        default: ns = 0;
      }
      
      return ns;
    }
    

    
    
  }


}
