/*  Copyright (C) 2013 Zoltán Nagy
 *  Created by Zoltán Nagy on 5/28/13.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

//   std includes
#include <cmath>

//   project headers
#include "bits/qcd-coupling.h"


namespace duct {
  
  
  double __qcd_coupling_msbar(double q2)
  {
    constexpr double thr[7] = {0.0, 0.0, 0.0, 0.0, 1.69, 20.25, 30625};
    constexpr double lmd[7] = {0.150787, 0.150787, 0.150787, 0.150787, 0.111104, 0.056656, 0.010287};
    
    unsigned nf = 6;
    while(q2 < thr[nf] && nf > 3) --nf;
    
    /*    computing the NLO alpha_s   */
    double b0 = beta0(nf);
    double t = 0.5*b0*log(q2/lmd[nf]);
    double ox, br = beta1(nf)/b0;
    double x = t+br, lbr = br*std::log(br)-br;
    
    do {
      ox = x;
      x -= x*(x-br*std::log(x)+lbr-t)/(x-br);
    } while(std::abs(ox-x)/ox > 1e-6);
    
    return 1.0/(x-br);
  }

  
  
}
