/*  Copyright (C) 2010-2013 Zoltán Nagy
 *  Created by Zoltán Nagy on 9/2/10.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
#include "bits/nlo-statistical-basis.h"
#include <limits>


namespace duct {
  
  void statistical_basis::insert_closed_string(const int *begin, const int *end)
  {
    statistical_basis& p = *this;
    const int *i;
    
    p[*begin].color.prev = *(end-1);
    p[*begin].color.next = *(begin+1);
    
    for(i = begin+1; i < end-1; i++) {
      p[*i].color.prev = *(i-1);
      p[*i].color.next = *(i+1);
    }
    
    p[*i].color.prev = *(i-1);
    p[*i].color.next = *begin;
  }
 
  void statistical_basis::insert_closed_stringP(const int *begin, const int *end)
  {
    statistical_basis& p = *this;
    const int *i;

    p[*begin].colorP.prev = *(end-1);
    p[*begin].colorP.next = *(begin+1);
    
    for(i = begin+1; i < end-1; i++) {
      p[*i].colorP.prev = *(i-1);
      p[*i].colorP.next = *(i+1);
    }
    
    p[*i].colorP.prev = *(i-1);
    p[*i].colorP.next = *begin;
  }
  
  void statistical_basis::insert_open_string(const int *begin, const int *end)
  {
    statistical_basis& p = *this;
    const int *i;

    p[*begin].color.prev = particle::colorlink::quarkend;
    p[*begin].color.next = *(begin+1);
    
    for(i = begin+1; i < end-1; i++) {
      p[*i].color.prev = *(i-1);
      p[*i].color.next = *(i+1);
    }
    
    p[*i].color.prev = *(i-1);
    p[*i].color.next = particle::colorlink::antiquarkend;
  }

  void statistical_basis::insert_open_stringP(const int *begin, const int *end)
  {
    statistical_basis& p = *this;
    const int *i;

    p[*begin].colorP.prev = particle::colorlink::quarkend;
    p[*begin].colorP.next = *(begin+1);
    
    for(i = begin+1; i < end-1; i++) {
      p[*i].colorP.prev = *(i-1);
      p[*i].colorP.next = *(i+1);
    }
    
    p[*i].colorP.prev = *(i-1);
    p[*i].colorP.next = particle::colorlink::antiquarkend;
  }
  
  
  std::ostream& operator<<(std::ostream& os, const statistical_basis& a) 
  {
    const model *mdl = a.get_model();

    //----- the ranges -----
    auto qcd = a.range<qcd_out>();
    auto non = a.range<non_qcd>();
    auto inc = a.range<incomings>();
    
    //------ print the momenta ----
    int idx = 1-a.number_of<non_qcd>() - a.number_of<incomings>();
    
    if(non.first != non.second) {
      os<<"+---- non-qcd outgoings -----\n";
      for(auto pi = non.first; pi != non.second; ++pi)
        os<<"|  "<<mdl->name(pi->flavor)<<"("<<idx++<<") : "
        <<pi->momentum<<"  "<<pi->momentum.mag2()<<std::endl;
    }
    
    os<<"+---- incomings -----\n";
    for(auto pi = inc.first; pi != inc.second; ++pi) {
      os<<"|  "<<mdl->name(-(pi->flavor))<<"("<<idx++<<") : "
      <<pi->momentum<<"  "<<pi->momentum.mag2()
      <<"\t   "<<pi->health
      <<"  "<<pi->color
      <<"  "<<pi->colorP
      <<std::endl;
    }
    
    os<<"+---- qcd outgoings -----\n";
    for(auto pi = qcd.first; pi != qcd.second; ++pi){
      os<<"|  "<<mdl->name(pi->flavor)<<"("<<idx++<<") : "
      <<pi->momentum<<"  "<<pi->momentum.mag2()
      <<"\t   "<<pi->health
      <<"  "<<pi->color
      <<"  "<<pi->colorP
      <<std::endl;
    }
    os<<"+----\n\n\n";
    
/*    
    //----- check + spinors -------
    idx = -a.number_of<non_qcd>()-1;
    
    if(non.first != non.second) {
      os<<"+---- non-qcd outgoings -----\n";
      for(auto pi = non.first; pi != non.second; ++pi)
        os<<"|  |"<<ptbl->name(pi->flavor)<<"("<<idx++<<") +> : "
        <<0.5*vector(dirac_adj(pi->plus), pi->plus)<<std::endl; 
    }
    
    os<<"+---- incomings-----\n";
    os<<"|  |"<<ptbl->name(a[-1].flavor)<<"("<<idx++<<") +> : "
    <<0.5*vector(dirac_adj(a[-1].plus), a[-1].plus)<<std::endl; 
    
    os<<"|  |"<<ptbl->name(a[0].flavor)<<"("<<idx++<<") +> : "
    <<0.5*vector(dirac_adj(a[0].plus), a[0].plus)<<std::endl; 
    
    os<<"+---- qcd outgoings -----\n";
    for(auto pi = qcd.first; pi != qcd.second; ++pi)
      os<<"|  |"<<ptbl->name(pi->flavor)<<"("<<idx++<<") +> : "
      <<0.5*vector(dirac_adj(pi->plus), pi->plus)<<std::endl; 
    os<<"+----\n\n\n";
    
    
    //----- check the - spinors -------
    idx = -a.number_of<non_qcd>()-1;
    
    if(non.first != non.second) {
      os<<"+---- non-qcd outgoings -----\n";
      for(auto pi = non.first; pi != non.second; ++pi)
        os<<"|  |"<<ptbl->name(pi->flavor)<<"("<<idx++<<") -> : "
        <<0.5*vector(dirac_adj(pi->minus), pi->minus)<<std::endl; 
    }
    
    os<<"+---- incomings -----\n";
    os<<"|  |"<<ptbl->name(a[-1].flavor)<<"("<<idx++<<") -> : "
    <<0.5*vector(dirac_adj(a[-1].minus), a[-1].minus)<<std::endl; 
    
    os<<"|  |"<<ptbl->name(a[0].flavor)<<"("<<idx++<<") -> : "
    <<0.5*vector(dirac_adj(a[0].minus), a[0].minus)<<std::endl; 
    
    os<<"+---- qcd outgoings -----\n";
    for(auto pi = qcd.first; pi != qcd.second; ++pi)
      os<<"|  |"<<ptbl->name(pi->flavor)<<"("<<idx++<<") -> : "
      <<0.5*vector(dirac_adj(pi->minus), pi->minus)<<std::endl; 
    os<<"+----\n\n\n";

    
    
    
    
    //----- print the + spinors -------
    idx = -a.number_of<non_qcd>()-1;
    
    if(non.first != non.second) {
      os<<"+---- non-qcd outgoings -----\n";
      for(auto pi = non.first; pi != non.second; ++pi)
        os<<"|  |"<<ptbl->name(pi->flavor)<<"("<<idx++<<") +> : "
        <<pi->plus<<"  "<<dirac_adj(pi->plus)*pi->plus<<std::endl; 
    }
    
    os<<"+---- incomings-----\n";
    os<<"|  |"<<ptbl->name(a[-1].flavor)<<"("<<idx++<<") +> : "
    <<a[-1].plus<<"  "<<dirac_adj(a[-1].plus)*a[-1].plus<<std::endl; 
    
    os<<"|  |"<<ptbl->name(a[0].flavor)<<"("<<idx++<<") +> : "
    <<a[0].plus<<"  "<<dirac_adj(a[0].plus)*a[0].plus<<std::endl; 
    
    os<<"+---- qcd outgoings -----\n";
    for(auto pi = qcd.first; pi != qcd.second; ++pi)
      os<<"|  |"<<ptbl->name(pi->flavor)<<"("<<idx++<<") +> : "
      <<pi->plus<<"  "<<dirac_adj(pi->plus)*pi->plus<<std::endl; 
    os<<"+----\n\n\n";
    
    
    //----- print the - spinors -------
    idx = -a.number_of<non_qcd>()-1;
    
    if(non.first != non.second) {
      os<<"+---- non-qcd outgoings -----\n";
      for(auto pi = non.first; pi != non.second; ++pi)
        os<<"|  |"<<ptbl->name(pi->flavor)<<"("<<idx++<<") -> : "
        <<pi->minus<<"  "<<dirac_adj(pi->minus)*pi->minus<<std::endl; 
    }
    
    os<<"+---- incomings -----\n";
    os<<"|  |"<<ptbl->name(a[-1].flavor)<<"("<<idx++<<") -> : "
    <<a[-1].minus<<"  "<<dirac_adj(a[-1].minus)*a[-1].minus<<std::endl; 

    os<<"|  |"<<ptbl->name(a[0].flavor)<<"("<<idx++<<") -> : "
    <<a[0].minus<<"  "<<dirac_adj(a[0].minus)*a[0].minus<<std::endl; 
    
    os<<"+---- qcd outgoings -----\n";
    for(auto pi = qcd.first; pi != qcd.second; ++pi)
      os<<"|  |"<<ptbl->name(pi->flavor)<<"("<<idx++<<") -> : "
      <<pi->minus<<"  "<<dirac_adj(pi->minus)*pi->minus<<std::endl; 
    os<<"+----\n\n\n";
    
  */
    auto itot = a.momentum_sum<incomings>();
    auto otot = a.momentum_sum<outgoings>();
    
    os<<"Total incoming momentum     : "<<itot<<std::endl;
    os<<"Total outgoing momentum     : "<<otot<<std::endl;
    os<<"Momentum conservation       : "<<itot-otot<<std::endl;
    os<<"\nColor supression index      : "<<a.color_suppression_index<<std::endl;
    
    return os;
  }
  
  
}


