/*  Copyright (C) 2013 Zoltán Nagy
 *  Created by Zoltán Nagy on 9/8/13.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
#include <string>
#include <utility>
#include <functional>
#include <memory>
#include <list>

#include <cstdlib>
#include <cstring>
#include <getopt.h>
#include <dlfcn.h>

#include "user.h"
#include "bits/nlo-c-support.h"
#include "main-mod.h"


/*     This is the main function. Now I can do this....   */
using namespace duct;



//----- print out the help -----
void main_add_help()
{
  std::cout
  <<"Usage: deductor --add my-module [-r dir] name1 [name2] [name3] ...\n\n"
  
  <<"Options of --add mode:\n"
  <<"  my-module            The module file contains the user defined functions.\n"
  <<"  -r directory         Save the result to this directory (default: ./result)\n"
  <<"--------------------------------------------------------------------------------\n"
  <<std::endl;
}



int main_add(int argc, char **argv)
{
  const option long_opt[] = {
    {0, 0, 0, 0}
  };
  
  //----- defoult parameters -----
  const char *module = argv[optind-1];
  std::string res("./result/");
  
  //----- start the parsing -----
  while(true) {
    int opt_idx = 0;
    int c = getopt_long(argc, argv, "r:", long_opt, &opt_idx);
    if(c == -1) break;
    
    switch(c) {
      case 'r': res = optarg; break;
      case '?': break;
      default: throw; break;
    }
  }
  
  if(res.back() != '/') res += '/';
  if(argc-optind <= 0) return 0;
  
  //----- opening the modul file -----
  void *handle = nullptr;
 
  /*   Collecting the users    */
  main_add_list *users = nullptr;
  
  //----- open the user defined file -----
  std::string bundle = std::string(module)+".bundle";
  std::string dir = bundle+"/output/";

  if(!(handle = ::dlopen((bundle+"/module"+__SOEXT__).c_str(), RTLD_LAZY))) {
    std::cerr<<"can't open the module "<<module<<"!\n"<<dlerror()<<std::endl;
    return 1;
  }
  
  //----- try to find the symbol table -----
  std::cout<<"Trying to find the main_add() function in "<<module<<" : ";
  users = reinterpret_cast<main_add_list *>(::dlsym(handle, "main_add"));
  
  if(users) std::cout<<"OK\n"<<std::endl;
  else {
    std::cout<<"FAIL\n"<<std::endl;
    std::cerr<<"There is no \'main_add_list main_add()\' function in the module. Check your user module!"<<std::endl;
    goto close_module;
  }
  
  if(users->empty()) {
    std::cerr<<"There is \'main_add_list main_add()\' function in the module, but nothing to do."<<std::endl;
    goto close_module;
  }
  
  
  /*    Collecting files and printing the results   */
  {
    unsigned int k, nf = (unsigned int)(argc-optind);
    char **flist = (char **) std::malloc((nf+1)*sizeof(char *));
    std::memset(flist, 0, (nf+1)*sizeof(char *));
    
    for(const auto& ui : *users) {
      for(k = 0; k < nf; k++) {
        std::string tmp = (dir+argv[k+optind]) + "-"+ui.first+".dat";
        flist[k] = (char *) std::realloc(flist[k], (1+tmp.size())*sizeof(char));
        std::strcpy(flist[k], tmp.c_str());
      }
      
      /*   creating the result directory   */
      auto dn = res+ui.first+"/";
      
      if(::__nlo_helper_mkdir(dn.c_str(), S_IRWXU | S_IRGRP | S_IXGRP| S_IROTH | S_IXOTH)) {
        std::cerr<<"Unable to make directory : "<<dn<<std::endl;
        continue;
      }

      /*   collect the files and print the result   */
      std::cout<<"Analyser \'"<<ui.first<<"\' has collected "
      <<(*ui.second)(dn.c_str(), (const char **) flist)<<" MC events."<<std::endl;
    }
    
    /*   cleaning up  */
    for(k = 0; k < nf; k++) std::free(flist[k]);
    std::free(flist);
  }

  /*   closing the module   */
  ::dlclose(handle);
  
  return 0;
  
close_module:
  /*   cleaning up  */
  ::dlclose(handle);
  
  return 1;
}
