/*  Copyright (C) 2013 Zoltán Nagy
 *  Created by Zoltán Nagy on 9/8/13.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

//   std includes
#include <cstdlib>
#include <cstring>
#include <getopt.h>
#include <iostream>
#include <functional>
#include <dlfcn.h>

//   project headers
#include "main-mod.h"
#include "bits/nlo-c-support.h"



  
//----- print out the help -----
void main_calc_help()
{
  std::cout
  <<"Usage: deductor --calculate my-module -n|--name name\n\n"
  
  <<"Options of --calculate mode:\n"
  <<"  my-module              The module file contains the user defined functions.\n"
  <<"  -n, --name name        Name of the run.\n"
  <<"--------------------------------------------------------------------------------\n"
  <<std::endl;
}

void blurb()
{
  std::cout
  <<"+-------------------------------------------------------------------------+\n"
  <<"|                       Deductor release 1.0.0                            |\n"
  <<"|                       Z. Nagy and D.E. Soper                            |\n"
  <<"|             A parton shower Monte Carlo event generator                 |\n"
  <<"|          not including an underlying event or hadronization             |\n"
  <<"|             http://www.desy.de/~znagy/Site/deductor.html                |\n"
  <<"|               http://pages.uoregon.edu/soper/deductor/                  |\n"
  <<"|                                                                         |\n"
  <<"| Please cite JJJJ nnnn (2014) ppp [arXiv:xxxx.yyyy] if you use this      |\n"
  <<"| package for scientific work.                                            |\n"
  <<"|                                                                         |\n"
  <<"| Deductor is provided without warranty under the terms of the GNU GPLv3. |\n"
  <<"| See COPYING file for details.                                           |\n"
  <<"+-------------------------------------------------------------------------+\n"
  <<std::endl;
}


int main_calc(int argc, char **argv)
{
  /*  message   */
  blurb();
  
  //----- long options -----
  option longopt[] = {
    {"name",     required_argument, 0, 'n' },
    {0, 0, 0, 0}
  };
  
  //------ default options -----
  const char *module = argv[optind-1];
  const char *name = "run";
  
  //------ argument parsing -----
  while(1) {
    int optidx = 0;
    int c = ::getopt_long(argc, argv, "n:", longopt, &optidx);
    if(c == -1) break;
    
    switch(c) {
      case 'n': name = optarg; break;
      case '?': break;
      default: throw; break;
    }
  }
  
  //----- user module -----
  std::cout
  <<"Name of the run    : "<<name<<"\n"
  <<"User module        : "<<module<<"\n"
   <<std::endl;
  
  /*   creating the output directory   */
  std::string bundle = std::string(module)+".bundle";
  std::string output = bundle+"/output";
 
  if(::__nlo_helper_mkdir(output.c_str(), S_IRWXU | S_IRGRP | S_IXGRP| S_IROTH | S_IXOTH)) {
    std::cerr<<"Unable to make directory : "<<output<<std::endl;
    throw "Unable to make the output directory";
  }
  
  //----- open the user defined file -----
  void *handle = ::dlopen((bundle+"/module"+__SOEXT__).c_str(), RTLD_LAZY);
  
  if(!handle) {
    std::cerr<<"can't open the module "<<module<<"!\n";
    throw dlerror();
  }
  
  /*    try to find the main function   */
  std::cout<<"Trying to find the main_calc() function : ";
  auto __main = reinterpret_cast<std::function<void(const char *, const char *)>*>(::dlsym(handle, "main_calc"));
  
  if(__main) {
    std::cout<<"OK\n"<<std::endl;
    __main->operator()(output.c_str(), name);
  } else {
    std::cout<<"FAIL\n"<<std::endl;
    std::cerr<<"There is no \'man_calc(const char *, const char *)\' function in the module. Check your user module!"<<std::endl;
  }

  //   close the dlopend files
  ::dlclose(handle);
  
  return 0;
}

