/*  Copyright (C) 2013 Zoltán Nagy
 *  Created by Zoltán Nagy on 12/21/13.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program. If not, see <http://www.gnu.org/licenses/>.
 */


//   std includes
#include <cstdlib>
#include <cstring>
#include <iostream>
#include <functional>
#include <dlfcn.h>

//   project headers
#include "main-mod.h"



//----- print out the help -----
void main_kern_help()
{
  std::cout
  <<"Usage: deductor --kernel my-module \n\n"
  
  <<"Options of --kernel mode:\n"
  <<"  my-module              The module file contains the user defined functions.\n"
  <<"--------------------------------------------------------------------------------\n"
  <<std::endl;
}

extern void blurb();


int main_kern(const char *module)
{
  /*  message   */
  blurb();
  
  //----- user module -----
  std::cout
  <<"User module : "<<module<<"\n"
  <<std::endl;
  
  
  std::string sofile = std::string(module)+".bundle/module"+__SOEXT__;
  void *handle = ::dlopen(sofile.c_str(), RTLD_LAZY);
 
  if (!handle) {
    std::cerr<<"can't open the module "<<module<<"!\n";
    throw dlerror();
  }
  
  /*    try to find the main function   */
  std::cout<<"Trying to find the main_kern() function : ";
  auto __main = reinterpret_cast<std::function<void()>*>(::dlsym(handle, "main_kern"));
  
  if(__main) {
    std::cout<<"OK\n"<<std::endl;
    __main->operator()();
  } else {
    std::cout<<"FAIL\n"<<std::endl;
    std::cerr<<"There is no \'main_kern()\' function in the module. Check your user module!"<<std::endl;
  }
  
  //   close the dlopend files
  ::dlclose(handle);
  
  
  return 0;
}

