/*  Copyright (C) 2013 Zoltán Nagy
 *  Created by Zoltán Nagy on 9/8/13.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

//   std includes
#include <cstdlib>
#include <iostream>
#include <getopt.h>



extern void main_calc_help();
extern void main_add_help();
extern void main_kern_help();
extern void main_mod_help();

extern int main_add(int, char **);
extern int main_calc(int, char **);
extern int main_kern(const char *);
extern int main_mod(int, char **);



int main(int argc, char **argv)
{
  option longopt[] = {
    {"add", required_argument, 0, 0},
    {"calculate", required_argument, 0, 0},
    {"kernel", required_argument, 0, 0},
    {"module", required_argument, 0, 0},
    {"help", no_argument, 0, 'h'},
    {0, 0, 0, 0}
  };
  
  try {
    while(1) {
      int optidx = 0;
      int c = getopt_long(argc, argv, "h", longopt, &optidx);
      if(c == -1) break;
      
      switch(c) {
        case 'h':
          main_mod_help();
          main_kern_help();
          main_calc_help();
          main_add_help();
          exit(0);
          break;
        case 0:
          switch(optidx) {
            case 0: main_add(argc, argv); break;
            case 1: main_calc(argc, argv); break;
            case 2: main_kern(optarg); break;
            case 3: main_mod(argc, argv); break;
          }
          break;
        case '?': break;
        default: throw; break;
      }
    }
  } catch(const char *message) {
    std::cerr<<"Error : "<<message<<std::endl;
    return 1;
  } catch(const std::exception& e) {
    std::cerr<<"Error : "<<e.what()<<std::endl;
    return 1;
  } catch(...) {
    std::cerr<<"Unexpected error!"<<std::endl;
    return 1;
  }
  
  return 0;
}
